"""
`Pyfx` TUI rendering related module.

It contains a homemade JSON widgets Library and collection of UI components to
interact with :class:`~pyfx.app.PyfxApp`.
"""
from .view_manager import View
from .view_config import ViewConfiguration
